/*
 *  procedures to deal the cards
 */

#include "p_poker.h"
#include "p_common.h"

/* a template of the unshuffled pack */
static playing_card standard[DECK_SIZE];

static playing_card deck[DECK_SIZE];
static int top_of_pack;

/* These are for the pack used to deal from */

long    rand(void);


/* This function initialises the deck of cards */

void
open_pack(playing_card *packptr)
{
	int     i, j, card;

	for (i = 0; i < 4; i++) {
		for (j = 1; j < 14; j++) {
			card = (i * 13) + j - 1;
			packptr[card].face_value = j;
			packptr[card].suit_value = i;

			switch (j) {
				case  1: packptr[card].face = 'A'; break;
				case 10: packptr[card].face = 'T'; break;
				case 11: packptr[card].face = 'J'; break;
				case 12: packptr[card].face = 'Q'; break;
				case 13: packptr[card].face = 'K'; break;
				default: packptr[card].face = '0' + j;
			}

			switch (i) {
				case 0: packptr[card].suit = 'H'; break;
				case 1: packptr[card].suit = 'C'; break;
				case 2: packptr[card].suit = 'S'; break;
				case 3: packptr[card].suit = 'D';
			}

		}
	}
	return;

}

/* create an unshuffled static pack */
void
initialise_cards(void)
{
	open_pack(standard);
}

/* create a new pack, that can be manipulated	 */
void
new_deal(void)
{
	int     i;
	for (i = 0; i < DECK_SIZE; i++)
		deck[i] = standard[i];
}

/* swap two cards around */
void
swap(playing_card *a, playing_card *b)
{
	playing_card temp;
	temp = *a;
	*a = *b;
	*b = temp;

}

/* This function will shuffle the deck that is used for manipulation	 */
void
shuffle_pack(void)
{
	int     i, j, swap_with;
	/*
	 * The pack is shuffled 500 times. The higher this value the more
	 * random is the pack, but it is slower between deals
	 */

	for (i = 0; i < 500; i++) {
		for (j = 0; j < DECK_SIZE; j++) {
			swap_with = rand() % DECK_SIZE;
			swap(&deck[j], &deck[swap_with]);
		}
	}
}

void
sort(playing_card *hand)
{
	int     i, j, smallest;

	for (i = 0; i < HAND_SIZE - 1; ++i) {
		smallest = i;
		for (j = (i + 1); j < HAND_SIZE; ++j)
			if ((hand[j].face_value < hand[smallest].face_value) ||
			    ((hand[j].face_value == hand[smallest].face_value)
			    && (hand[j].suit < hand[smallest].suit)))
				smallest = j;

		if (smallest != i)
			swap((hand + i), (hand + smallest));

	}
}

/* deal out hands to YOU & ME from deck , starting at top_of_pack */
void
deal_two_hands(playing_card *you, playing_card *me)
{
	int     i;

	shuffle_pack();
	top_of_pack = 0;

	for (i = 0; i < HAND_SIZE; top_of_pack += 2, i++) {
		you[i] = deck[top_of_pack];
		me[i]  = deck[top_of_pack + 1];
	}
	sort(you);
	sort(me);
}

/*
 * overwrite the cards that a player wants to change, by taking cards
 * from top of deck
 */
void
re_deal(playing_card *hand, describe needs)
{
	int     i;
	for (i = 0; i < needs.reject; i++)
		hand[needs.exchange[i]] = deck[top_of_pack++];

	sort(hand);
}
