/*
 * Copyright (c) 1997 Hong Hunsoo of (KAIST)
 * The Korea Advanced Institute of Science and Technology
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Neither the name of the KAIST nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "common.h"
#include "xcommon.h"
#include "f_fonts.h"
#include "f_inter.h"
#include "ps_italic.h"

#define LAST_NO 4
#define LAST_SZ 700

static Pixmap pix0, pix1, pixs[LAST_NO];
static int currx, curry;
static struct font_t *fp;

static void	prepare_first(void);
static void	prepare_last(void);
static void	prepare_lastlast(void);
static int	quit_check(void);

void
px_first(void)
{
	XEvent event;
	char	input[10];
	int	no;
	KeySym mykey;
	int	done;

	fp = assemble(ps_italic_chars);
	prepare_first();

	done = 1;
	while (done) {
		XNextEvent(dpy, &event);

		switch (event.type) {
		case Expose:
			XGetGeometry(dpy, appwin, &dummy_win, &app_x, &app_y,
				&app_width, &app_height, &dummy, &app_depth);
			currx = 0; curry = 0;
			XCopyArea(dpy, pix0, appwin, gc,
				0, 0, app_width, app_height, 0, 0);
			break;
		case KeyPress:
			no = XLookupString((XKeyEvent *) &event,
				input, 10, &mykey, 0);
			if (no == 1 && input[0] == 'q')
				exit(0);
			XClearArea(dpy, appwin, 0, 0, app_width, app_height, 0);
			XFreePixmap(dpy, pix0); /* Free */
			done = 0;
			break;

		}
	}
	return;
}

int
predicate(Display *dpy, XEvent *ep, char *p)
{
	if (ep->type == KeyPress)
		return 1;
	return 0;
}

void
px_last(void)
{
	int	i, bottom, final;
	int	onthis, onthat, this, that;
	
	XGetGeometry(dpy, appwin, &dummy_win, &app_x, &app_y,
		&app_width, &app_height, &dummy, &app_depth);

	prepare_last();

	final = LAST_NO * LAST_SZ - app_height;
	for (i=0; i < final; i += 5) {
		bottom = i + app_height;
		this = i / LAST_SZ;
		that = bottom / LAST_SZ;

		if (quit_check())
			exit(1);

		if (this == that) {
			XCopyArea(dpy, pixs[this], appwin, gc,
				0, i % LAST_SZ, app_width, app_height, 0, 0);
		} else {
			onthat = i % LAST_SZ;
			onthis = LAST_SZ - onthat;

			XCopyArea(dpy, pixs[this], appwin, gc, 0, onthat,
				app_width, onthis, 0, 0);
			XCopyArea(dpy, pixs[that], appwin, gc, 0, 0,
				app_width, onthat, 0, onthis);
		}
	}
	prepare_lastlast();
	XCopyArea(dpy, pix1, appwin, gc, 0, 0, app_width, app_height, 0, 0);
	XFlush(dpy);
	sleep(5);
}

static int
quit_check(void)
{
	XEvent event;
	char	input[10];
	int	no;
	KeySym mykey;

	while (XPending(dpy)) {
		XNextEvent(dpy, &event);

		switch (event.type) {
			case KeyPress:
				no = XLookupString((XKeyEvent *) &event,
					input, 10, &mykey, 0);
				if (no == 1 && input[0] == 'q')
					return 1;
				break;
		}
	}
	return 0;
}

static void
prepare_first(void)
{
	int	height;

	height = app_height * 2;
	if (pix0)
		XFreePixmap(dpy, pix0);

	pix0 = XCreatePixmap(dpy, appwin, app_width, height, app_depth);

	gc_invert(gc);
	XFillRectangle(dpy, pix0, gc, 0, 0, app_width, height);
	gc_invert(gc);

	xstr_cput(dpy, pix0, gc, app_width,  250, 250, "KAIST", fp);
	xstr_cput(dpy, pix0, gc, app_width,  400, 150, "gambling", fp);
	xstr_cput(dpy, pix0, gc, app_width,  550, 150, "Area", fp);

	return;
}

static void
prepare_last(void)
{
	int	i, wid, hei;
	char	**cpp;
	static char *msgs[] = {
		"KAIST", "gambling",
		"Area", "by",
		"Hong Hunsoo",
		"Screenplayed by", "Hong Hunsoo", "",
		"First Played by", "Hong Hunsoo", "",
		"X Window program trainee", "Hong Hunsoo",
		" ",
		"Directed by", "Hong Hunsoo",
		"(c)"
	};

	hei = LAST_SZ;
	wid = app_width;

	gc_invert(gc);
	for (i = 0; i < (LAST_NO-1); i++) {
		pixs[i] = XCreatePixmap(dpy, appwin,
			wid, hei, app_depth);
		XFillRectangle(dpy, pixs[i], gc, 0, 0, wid, hei);
	}
	pix1 = XCreatePixmap(dpy, appwin, wid, hei, app_depth);
	XFillRectangle(dpy, pix1, gc, 0, 0, wid, hei);

	pixs[LAST_NO-1] = pixs[0];
	gc_invert(gc);

	i = 0;
	cpp = msgs;
	if (++i >= (LAST_NO-1))
		return;
	xstr_cput(dpy, pixs[i], gc, wid,  200, 150, *cpp++, fp);
	xstr_cput(dpy, pixs[i], gc, wid,  300, 100, *cpp++, fp);
	xstr_cput(dpy, pixs[i], gc, wid,  400, 100, *cpp++, fp);
	xstr_cput(dpy, pixs[i], gc, wid,  590, 80, *cpp++, fp);
	xstr_cput(dpy, pixs[i], gc, wid,  680, 80, *cpp++, fp);

	set_gclinewidth(gc, 0);
	if (++i >= (LAST_NO-1))
		return;
	xstr_cput(dpy, pixs[i], gc, wid,  200, 50, *cpp++, fp);
	xstr_cput(dpy, pixs[i], gc, wid,  250, 50, *cpp++, fp);
	xstr_cput(dpy, pixs[i], gc, wid,  300, 50, *cpp++, fp);
	xstr_cput(dpy, pixs[i], gc, wid,  350, 50, *cpp++, fp);
	xstr_cput(dpy, pixs[i], gc, wid,  400, 50, *cpp++, fp);
	xstr_cput(dpy, pixs[i], gc, wid,  450, 50, *cpp++, fp);
	xstr_cput(dpy, pixs[i], gc, wid,  500, 50, *cpp++, fp);
	xstr_cput(dpy, pixs[i], gc, wid,  550, 50, *cpp++, fp);
	xstr_cput(dpy, pixs[i], gc, wid,  600, 50, *cpp++, fp);
	xstr_cput(dpy, pixs[i], gc, wid,  630, 50, *cpp++, fp);
	xstr_cput(dpy, pixs[i], gc, wid,  680, 50, *cpp++, fp);

	return;
}

static void
prepare_lastlast(void)
{
	int	wid;
	char	**cpp;
	static char *msgs[] = {
		"Copyright (c) 1997 Hong Hunsoo of (KAIST)",
		"The Korea Advanced Institute of Science and Technology",
		"All rights reserved.",
	};

	wid = app_width;
	fprintf(stderr, "%s() started.\n", __F);

	cpp = msgs;
	set_gclinewidth(gc, 0);
	xstr_cput(dpy, pix1, gc, wid,  360, 20, *cpp++, fp);
	xstr_cput(dpy, pix1, gc, wid,  380, 20, *cpp++, fp);
	xstr_cput(dpy, pix1, gc, wid,  400, 20, *cpp++, fp);
	fprintf(stderr, "%s() ended.\n", __F);

	return;
}
