/*
 * Copyright (c) 1997 Hong Hunsoo of (KAIST)
 * The Korea Advanced Institute of Science and Technology
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Neither the name of the KAIST nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xmu/Xmu.h>

#include "common.h"
#include "xcommon.h"
#include "p_poker.h"
#include "p_common.h"
#include "p_window.h"
#include "cards.h"

Window	rootwin;
Window	appwin, dummy_win;
int	screenno;
Display	*dpy;

unsigned long pixel_white, pixel_black;
unsigned long pixel_red, pixel_green, pixel_blue;
GC	gc, gcr, gc_red, gc_white, gc_black;
char	line[256];
int	win_x, win_y;
unsigned int win_height, win_width, win_borderWidth, win_depth;
int	app_x, app_y;
unsigned int app_width, app_height, app_borderWidth, app_depth;
int	dummy;

void	do_event(void);
void	for_expose(void);
int	offset5[5];

#define init
#define MSG_NO 50

struct msg_t {
	int	m_ycoor;
	char	*m_msg;
};

struct msg_t msgs[MSG_NO], *msgt;

int
px_initial(void)
{
	int	i;
	XGCValues xgcv;
	Colormap cmp;
	unsigned long mask;
	Font	font;


	dpy = XOpenDisplay(NULL);
	XSetErrorHandler(error);

	rootwin   = XDefaultRootWindow(dpy);
	screenno  = XDefaultScreen(dpy);
	cmp	  = XDefaultColormap(dpy, screenno);

	pixel_black = XBlackPixel(dpy, screenno);
	pixel_white = XWhitePixel(dpy, screenno);
	pixel_red   = name_to_pixel(cmp, "red");
	pixel_green = name_to_pixel(cmp, "green");
	pixel_blue  = name_to_pixel(cmp, "blue");

	XGetGeometry(dpy, rootwin, &dummy_win, &win_x, &win_y,
		&win_width, &win_height, &win_borderWidth, &win_depth);
#if TMP
	appwin = XCreateSimpleWindow(dpy, rootwin, 100, 100, 500, 700, 0,
		pixel_black, pixel_blue);
#else
	appwin = XCreateSimpleWindow(dpy, rootwin, 0, 0,
		900, 600, 0, pixel_black, pixel_blue);
#endif
	XGetGeometry(dpy, appwin, &dummy_win, &app_x, &app_y,
		&app_width, &app_height, &app_borderWidth, &app_depth);
	XSelectInput(dpy, appwin, ExposureMask |KeyPressMask);;

	font = XLoadFont(dpy,
		"-adobe-courier-bold-r-normal--18-180-75-75-m-110-iso8859-1");
	xgcv.font = font;
	xgcv.function = GXcopy;
	xgcv.line_width = 3;
	xgcv.fill_style = FillSolid;
	xgcv.join_style = JoinRound;
	xgcv.fill_rule = WindingRule;
	xgcv.foreground = pixel_green;
	xgcv.background = pixel_blue;

	mask = GCFunction | GCForeground | GCBackground | GCFillRule
		| GCFillStyle | GCLineWidth | GCFont | GCJoinStyle;

	gc = XCreateGC(dpy, appwin, mask, &xgcv);

	XSetGraphicsExposures(dpy, gc, True);
	xgcv.foreground = pixel_blue;
	xgcv.background = pixel_green;
	gcr = XCreateGC(dpy, appwin, mask, &xgcv);

	xgcv.foreground = pixel_red;
	xgcv.background = pixel_green;
	gc_red = XCreateGC(dpy, appwin, mask, &xgcv);

	xgcv.foreground = pixel_white;
	xgcv.background = pixel_green;
	gc_white = XCreateGC(dpy, appwin, mask, &xgcv);

	xgcv.foreground = pixel_black;
	xgcv.background = pixel_green;
	gc_black = XCreateGC(dpy, appwin, mask, &xgcv);

	XMapWindow(dpy, appwin);
	card_init(dpy, screenno);

	XFlush(dpy); /* XXX */

	for (i=0; i < 5; i++) {
		offset5[i] = 10 + (CARD_WIDTH + 5) * i;
	}
	for (i=0; i<MSG_NO; i++) {
		msgs[i].m_msg = (char *) malloc(100);
		if (msgs[i].m_msg == NULL)
			err("%s(): msgs[%d].m_msg == NULL", __F, i);
		msgs[i].m_ycoor = (i+1) * 20;
	}

	msgt = msgs;
	return 0;
}

void
px_clear_mid(void)
{
	int	i, offset;

	offset = (CARD_WIDTH  + 10) * 5;

	XClearArea(dpy, appwin,
		offset, 0, app_width - offset, app_height, 0);
	for (i=0; i<MSG_NO; i++)
		msgs[i].m_msg[0] = 0;
	msgt = msgs;
}

void
px_clear_my_hand(playing_card *hand)
{
	int     i;
	int	facev, suitv;

	for (i = 0; i < HAND_SIZE; i++) {
		facev = hand[i].face_value;
		suitv = hand[i].suit_value;

		put_card(appwin, 4, 0, offset5[i], 200);
	}
}

void
px_show_hand(playing_card *hand, int whose)
{
	int     i, pos;
	int	facev, suitv;

	pos = whose ? 200 : 30;

	for (i = 0; i < HAND_SIZE; i++) {
		facev = hand[i].face_value;
		suitv = hand[i].suit_value;

		put_card(appwin, suitv, facev-1, offset5[i], pos);
	}
}

void
px_inform_ante(int pot) /* inform user that ante is required */
{
	px_clear_mid();
	px_putmsg("We ante%s. ", (pot > 10) ? " again" : "");
}

void
px_money(int you, int me, int pot)
{
	XClearArea(dpy, appwin, 10, 400, 150, 100, 0);
	sprintf(line, "Pot   = %3d", pot);
	XDrawString(dpy, appwin, gc, 10, 420, line, strlen(line));
	sprintf(line, "Yours = %4d", you);
	XDrawString(dpy, appwin, gc, 10, 440, line, strlen(line));
	sprintf(line, "Mine  = %4d", me);
	XDrawString(dpy, appwin, gc, 10, 460, line, strlen(line));
	XFlush(dpy);
}

void
px_show_turn(void)
{
	/* void */
}

int
px_choose_SDB(int source, int *amount)
{
	char    choice, *res;
	int     response;

	px_putmsg("Do you wish to Stay, Drop or Bet:");
	XFlush(dpy);

	get_SDB(source, &choice);
	res = choice == 's' ? "Stay!" : choice == 'd' ? "Drop!!" :  "Bet!";
	px_putmsg(res);

	if (choice == 'b')
		px_show_inc(0, source, amount);

	switch (choice) {
		case 's': /* Staying */
			response = 1;
			break;
		case 'd': /* dropping */
			response = 2;
			break;
		case 'b': /* betting */
			response = (*amount / 5) + 3;	/* bet this */

	}
	return response;
}

void
px_get_input(int des, char *val)
{
	XEvent event;
	KeySym mykey;
	int	no;
	char	buf[4];

	do {
		XNextEvent(dpy, &event);

		switch (event.type) {
		case Expose:
			for_expose();
			px_show_turn();
		case KeyPress:
			no = XLookupString((XKeyEvent *) &event,
				buf, 4, &mykey, 0);
			if (no == 1 && buf[0] == 'q')
				really();
			break;
		}
		*val = buf[0];
	} while (*val == '\022');
}

void
px_i_do(int what)
{
	char	*p;

	sprintf(line, "I will ");
	p = line + 7;

	switch (what) {
		case 1: sprintf(p, "stay.");		break;
		case 2: sprintf(p, "drop my cards.");break;
		case 3: sprintf(p, "call.");		break;
		case 4: case 5: case 6: case 7: case 8:
			sprintf(p, "bet %2d.", (5 * (what - 3))); break;
		default: sprintf(p, "raise %2d.", (5 * (what - 8))); break;
	}
	px_putmsg(line);
}

void
px_show_inc(int raising, int source, int *amount)
{
	px_putmsg("How much do you want to %s (1-5):",
	       raising ? "raise" : "bet");
	get_increment(source, amount);
	px_putmsg("%2d", *amount);
}

void
px_i_take(int num)
{
	px_putmsg("I will take %d card%s", num, (num != 1) ? "s." : ".");
}

void
px_delete_line(void)
{
}

void
px_show_choice(char choice, int count)
{
	char	buf[10];
	sprintf(buf, "%c ", choice);
	px_appendmsg(buf);
	XFlush(dpy);
}

void
px_show_value(char *owner, int value)
{
	px_putmsg("---%s card was %s!!", owner, value_to_string(value));
}

void
px_show_winner(int win)
{
	if (win == -1)
		px_putmsg("It is a draw, the pot stays. ");
	else
		px_putmsg(">>>>>  %s win!! <<<<<<", win ? "I" : "You");
}

/*
 * declare the over all winner, when all of a stake is lost, or no more
 * games are required
 */
void
px_show_over_win(int win)
{
	px_clear_mid();
	if (win == -1)
		px_putmsg("It is a draw.");
	else {
		px_putmsg("%sthe over all winner of that game.",
		       win ? "I am " : "You are ");
		if (win)
			px_putmsg("Now isn't that a pity!!");
		else {
			px_putmsg("CONGRATULATIONS!");
			px_putmsg("But that was only ONE game.");
		}
	}
}

/* ask if another game is requred, and get a y/n answer */
int
px_ano_yes_no(int source)
{
	char    reply;

	px_putmsg("Do you want another go? ");
	do
		px_get_input(source, &reply);
	while (reply != 'y' && reply != 'n')
		/* void */ ;

	return (reply == 'y');
}


void
px_putmsg(const char *fmt, ...)
{
	va_list cp;
	va_start(cp, fmt);

	vsprintf(msgt->m_msg, fmt, cp);
	XDrawString(dpy, appwin, gc, 450, msgt->m_ycoor,
		msgt->m_msg, strlen(msgt->m_msg));

	msgt++;
	XFlush(dpy);
	va_end(cp);
}

void
px_appendmsg(const char *string)
{
	strcat(msgt->m_msg, string);
	XDrawString(dpy, appwin, gc, 450, msgt->m_ycoor,
		msgt->m_msg, strlen(msgt->m_msg));

	XFlush(dpy);
}

void
px_die(void)
{
	px_last();
	exit(0);
}

void
px_delete_prev(void)
{
	msgt--;
	XClearArea(dpy, appwin, 450, msgt->m_ycoor - 14,
		app_width - 450, msgt->m_ycoor + 4, 0);
	msgt->m_msg[0] = 0;

	XFlush(dpy);
}

void
px_deletemsg(void)
{
	XClearArea(dpy, appwin, 450, msgt->m_ycoor - 14,
		app_width - 450, msgt->m_ycoor + 4, 0);
	msgt->m_msg[0] = 0;

	XFlush(dpy);
}

void
px_exposemsgs(void)
{
	int	i;
	char	*p;
	struct msg_t *msg;

	for (i=0; i<MSG_NO; i++) {
		msg = &msgs[i];
		p = msg->m_msg;
		if (p[0])
			XDrawString(dpy, appwin, gc, 450, msg->m_ycoor,
				msg->m_msg, strlen(msg->m_msg));
	}
	return;
}
